@@ -6,6 +6,24 @@ from pocomc.input_validation import assert_array_float, assert_array_within_inte
 
 
 class Reparameterise:
+    """
+    Class that reparameterises the model using change-of-variables parameter transformations.
+
+    Parameters
+    ----------
+    ndim : int
+        Dimensionality of sampling problem
+    bounds : array or list or None
+        Parameter bounds
+    periodic : list
+        List of indices corresponding to parameters with periodic boundary conditions
+    reflective : list
+        List of indices corresponding to parameters with periodic boundary conditions
+    scale : bool
+        Rescale parameters to zero mean and unit variance (default is true)
+    diagonal : bool
+        Use diagonal transformation (i.e. ignore covariance) (default is true)
+    """
     def __init__(self,
                  ndim: int,
                  bounds: Union[np.ndarray, list] = None,
@@ -13,24 +31,7 @@ class Reparameterise:
                  reflective: List[int] = None,
                  scale: bool = True,
                  diagonal: bool = True):
-        """
-        Function that reparameterises the model using change-of-variables parameter transformations.
 
-        Parameters
-        ----------
-        ndim : int
-            Dimensionality of sampling problem
-        bounds : array or list or None
-            Parameter bounds
-        periodic : list
-            List of indices corresponding to parameters with periodic boundary conditions
-        reflective : list
-            List of indices corresponding to parameters with periodic boundary conditions
-        scale : bool
-            Rescale parameters to zero mean and unit variance (default is true)
-        diagonal : bool
-            Use diagonal transformation (i.e. ignore covariance) (default is true)
-        """
         self.ndim = ndim
 
         if bounds is None:
@@ -49,8 +50,8 @@ class Reparameterise:
         self.sigma = None
         self.cov = None
         self.L = None
-        self.Linv = None
-        self.logdetL = None
+        self.L_inv = None
+        self.log_det_L = None
         self.scale = scale
         self.diagonal = diagonal
 
@@ -59,8 +60,10 @@ class Reparameterise:
     def apply_boundary_conditions(self, x: np.ndarray):
         """
         Apply boundary conditions (i.e. periodic or reflective) to input.
-        TODO add short description.
-        
+        The first kind include phase parameters that might be periodic
+        e.g. on a range ``[0,2*np.pi]``. The latter can arise in cases
+        where parameters are ratios where ``a/b`` and  ``b/a`` are equivalent.
+
         Parameters
         ----------
         x : np.ndarray
@@ -82,7 +85,8 @@ class Reparameterise:
     def _apply_periodic_boundary_conditions(self, x: np.ndarray):
         """
         Apply periodic boundary conditions to input.
-        TODO add short description.
+        This can be useful for phase parameters that might be periodic
+        e.g. on a range ``[0,2*np.pi]``
         
         Parameters
         ----------
@@ -105,8 +109,8 @@ class Reparameterise:
 
     def _apply_reflective_boundary_conditions(self, x: np.ndarray):
         """
-        Apply reflective boundary conditions to input.
-        TODO add short description.
+        Apply reflective boundary conditions to input. This can arise in cases
+        where parameters are ratios where ``a/b`` and  ``b/a`` are equivalent.
         
         Parameters
         ----------
@@ -124,7 +128,7 @@ class Reparameterise:
                     while x[j, i] > self.high[i]:
                         x[j, i] = self.high[i] - x[j, i] + self.high[i]
                     while x[j, i] < self.low[i]:
-                        x[j, i] = self.low[i] + self.low[i] - self.x[j, i]
+                        x[j, i] = self.low[i] + self.low[i] - x[j, i]
 
         return x
 
@@ -146,8 +150,8 @@ class Reparameterise:
         else:
             self.cov = np.cov(u.T)
             self.L = np.linalg.cholesky(self.cov)
-            self.Linv = np.linalg.inv(self.L)
-            self.logdetL = np.linalg.slogdet(self.L)[1]
+            self.L_inv = np.linalg.inv(self.L)
+            self.log_det_L = np.linalg.slogdet(self.L)[1]
 
     def forward(self, x: np.ndarray):
         """
@@ -182,17 +186,17 @@ class Reparameterise:
         -------
         x : np.ndarray
             Transformed input data
-        logdetJ : np.array
+        log_det_J : np.array
             Logarithm of determinant of Jacobian matrix transformation.
         """
         if self.scale:
-            x, logdetJ = self._inverse_affine(u)
-            x, logdetJ_prime = self._inverse(x)
-            logdetJ += logdetJ_prime
+            x, log_det_J = self._inverse_affine(u)
+            x, log_det_J_prime = self._inverse(x)
+            log_det_J += log_det_J_prime
         else:
-            x, logdetJ = self._inverse(u)
+            x, log_det_J = self._inverse(u)
 
-        return x, logdetJ
+        return x, log_det_J
 
     def _forward(self, x: np.ndarray):
         """
@@ -227,7 +231,7 @@ class Reparameterise:
         -------
         x : np.ndarray
             Transformed input data
-        logdetJ : np.array
+        log_det_J : np.array
             Logarithm of determinant of Jacobian matrix transformation.
         """
         x = np.empty(u.shape)
@@ -238,9 +242,9 @@ class Reparameterise:
         x[:, self.mask_right], J[:, self.mask_right] = self._inverse_right(u)
         x[:, self.mask_both], J[:, self.mask_both] = self._inverse_both(u)
 
-        logdetJ = np.array([np.linalg.slogdet(Ji * np.identity(len(Ji)))[1] for Ji in J])
+        log_det_J = np.array([np.linalg.slogdet(Ji * np.identity(len(Ji)))[1] for Ji in J])
 
-        return x, logdetJ
+        return x, log_det_J
 
     def _forward_affine(self, x: np.ndarray):
         """
@@ -257,7 +261,7 @@ class Reparameterise:
         if self.diagonal:
             return (x - self.mu) / self.sigma
         else:
-            return np.array([np.dot(self.Linv, xi - self.mu) for xi in x])
+            return np.array([np.dot(self.L_inv, xi - self.mu) for xi in x])
 
     def _inverse_affine(self, u: np.ndarray):
         """
@@ -276,11 +280,11 @@ class Reparameterise:
         """
         if self.diagonal:
             J = self.sigma
-            logdetJ = np.linalg.slogdet(J * np.identity(len(J)))[1]
-            return self.mu + self.sigma * u, logdetJ * np.ones(len(u))
+            log_det_J = np.linalg.slogdet(J * np.identity(len(J)))[1]
+            return self.mu + self.sigma * u, log_det_J * np.ones(len(u))
         else:
             x = self.mu + np.array([np.dot(self.L, ui) for ui in u])
-            return x, self.logdetL * np.ones(len(u))
+            return x, self.log_det_L * np.ones(len(u))
 
     def _forward_left(self, x: np.ndarray):
         """
@@ -411,7 +415,7 @@ class Reparameterise:
         -------
         x : np.ndarray
             Transformed input data
-        logdetJ : np.array
+        log_det_J : np.array
             Logarithm of determinant of Jacobian matrix transformation.
         """
         return u[:, self.mask_none], np.ones(u.shape)[:, self.mask_none]
